
//===========================================
`timescale 1ns/1ps

`include "./usb_host/usb_define.v"
module tb;

reg clk;
reg reset;

reg [1:0]       PHY_LINESTATE ;
reg             PHY_CLKOUT  ;
reg             PHY_RESET   ;
reg             PHY_TXREADY ;
reg             PHY_RXVALID ;
reg             PHY_RXACTIVE;
reg             PHY_RXERROR ;
reg  [7:0]      PHY_DATAIN;
wire [7:0]      PHY_DATAOUT;





wire                     usb_txpop;// usb -- >
wire                     usb_txact;// usb -- >
wire [7:0]               usb_rxdat;// usb -- >
wire                     usb_rxval;// usb -- >
wire                     usb_rxact;// usb -- >
wire [3:0]               endpt_sel;// usb -- >
wire                     usb_txcork;// -- >usb
wire [7:0]               usb_txdat;// -- >usb
wire                     usb_rxrdy;// -- >usb
wire [9:0]               descrom_raddr;
wire [7:0]               descrom_rdat;
wire [9:0]               desc_dev_addr;
wire [7:0]               desc_dev_len;
wire [9:0]               desc_qual_addr;
wire [7:0]               desc_qual_len;
wire [9:0]               desc_fscfg_addr;
wire [7:0]               desc_fscfg_len;
wire [9:0]               desc_hscfg_addr;
wire [7:0]               desc_hscfg_len;
wire [9:0]               desc_otherspeed_addr;
wire [7:0]               desc_strlang_addr;
wire [9:0]               desc_strvendor_addr;
wire [7:0]               desc_strvendor_len;
wire [9:0]               desc_strproduct_addr;
wire [7:0]               desc_strproduct_len;
wire [9:0]               desc_strserial_addr;
wire [7:0]               desc_strserial_len;
wire                     descrom_have_strings;
reg  clk_in;
wire clk_60;
wire [7:0] host_utmi_data_out  ;
wire       host_utmi_txvalid   ;
wire [1:0] host_utmi_op_mode   ;
wire [1:0] host_utmi_xcvrselect;
wire       host_utmi_termselect;
wire       host_utmi_dppulldown;
wire       host_utmi_dmpulldown;
wire [7:0] host_utmi_data_in   ;
wire       host_utmi_txready   ;
wire       host_utmi_rxvalid   ;
wire       host_utmi_rxactive  ;
wire       host_utmi_rxerror   ;
wire [1:0] host_utmi_linestate ;
reg [15:0] data_len;
reg [7:0]  token_pid;
reg [6:0]  token_dev;
reg [3:0]  token_ep;
reg        transfer_start_q;
reg        sof_transfer_q;
wire       tx_pop;
reg [7:0]  tx_data;
reg [7:0]  set_addr [0:7];
reg [7:0]  set_line_coding [0:14];
reg [7:0]  line_coding_data [0:6];
reg [7:0]  get_line_coding [0:7];
reg [7:0]  send_data [0:38];
reg        in_transfer;
wire       rx_push;
wire [7:0] rx_data;
wire       rx_done;
wire       tx_done;

integer    i;
integer    e;

//==============================================================
//======Clock 48MHz
initial begin
    clk_in= 0;
    reset = 1;
#100000
    reset = 0;
end
always #10 clk_in <= ~clk_in;

Gowin_PLLVR u_pll(
    .clkout(clk_60), //output clkout
    .clkin(clk_in) //input clkin
);


//==============================================================
//======USB Host Controller
initial begin
    transfer_start_q <= 1'b0;
    sof_transfer_q   <= 1'b0;
    in_transfer <= 1'b0;
    data_len = 16'd8;
#120000
//======Set USB Device Addr
    wait(!reset);
    usb_host_send(`PID_SETUP, 7'd0, 4'd0);
    data_len = 16'd8;
    for(i=0;i<8;i=i+1) begin
        tx_data = set_addr[i];
        wait(tx_pop);
        @(posedge clk_60);
        @(posedge clk_60);
    end
#2000
        in_transfer = 1'b1;
        usb_host_send(`PID_IN, 7'h0, 4'd0);
        #1600000;
#8000
//======Setup USB CDC Line Coding
    in_transfer = 1'b0;
    data_len = 16'd8;
    usb_host_send(`PID_SETUP, 7'h40, 4'd0);
    for(i=0;i<8;i=i+1) begin
        tx_data = set_line_coding[i];
        wait(tx_pop);
        @(posedge clk_60);
        @(posedge clk_60);
    end
    $display("\n Set Line Coding ");
#8000
//======Send USB CDC Line Coding Data
    in_transfer = 1'b0;
    data_len = 16'd7;
    usb_host_send(`PID_OUT, 7'h40, 4'd0);
    for(i=0;i<7;i=i+1) begin
        tx_data = line_coding_data[i];
        wait(tx_pop);
        @(posedge clk_60);
        @(posedge clk_60);
    end
    $display("\n Set Line Coding Data");
#8000
#8000
    //Endpoint 2 OUT
    in_transfer = 1'b0;
    data_len = 16'd38;
    usb_host_send(`PID_OUT, 7'h40, 4'd2);
    for(i=0;i<38;i=i+1) begin
        tx_data = send_data[i];
        wait(tx_pop);
        @(posedge clk_60);
        @(posedge clk_60);
    end
#8000
#8000
    //Endpoint 2 IN
    in_transfer = 1'b1;
    usb_host_send(`PID_IN, 7'h40, 4'd2);
    i=0;
    while(!rx_done) begin
            wait(rx_push);
            if (rx_data == send_data[i]) begin
                $display("\n RX Data %d == TX Data %d",rx_data,send_data[i]);
            end
            else begin
                $display("\n RX Data %d == TX Data %d",rx_data,send_data[i]);
                $display("\n IN/OUT test failed");
                $stop;
            end
            i=i+1;
            @(posedge clk_60);
            @(posedge clk_60);
    end
    $display("\n IN/OUT test passed");
#8000
#80000
$stop;
end

initial begin
    set_addr[0] = 8'h00;
    set_addr[1] = 8'h05;
    set_addr[2] = 8'h01;
    set_addr[3] = 8'h00;
    set_addr[4] = 8'h00;
    set_addr[5] = 8'h00;
    set_addr[6] = 8'h00;
    set_addr[7] = 8'h00;
end
initial begin
    set_line_coding[0] = 8'h21;
    set_line_coding[1] = 8'h20;
    set_line_coding[2] = 8'h00;
    set_line_coding[3] = 8'h00;
    set_line_coding[4] = 8'h00;
    set_line_coding[5] = 8'h00;
    set_line_coding[6] = 8'h07;
    set_line_coding[7] = 8'h00;
end
initial begin
    line_coding_data[0] = 8'h00;
    line_coding_data[1] = 8'hf4;
    line_coding_data[2] = 8'h01;
    line_coding_data[3] = 8'h00;
    line_coding_data[4] = 8'h00;
    line_coding_data[5] = 8'h00;
    line_coding_data[6] = 8'hff;
end
initial begin
    get_line_coding[0] = 8'hA1;
    get_line_coding[1] = 8'h21;
    get_line_coding[2] = 8'h00;
    get_line_coding[3] = 8'h00;
    get_line_coding[4] = 8'h00;
    get_line_coding[5] = 8'h00;
    get_line_coding[6] = 8'h07;
    get_line_coding[7] = 8'h00;
end

initial begin
    send_data[0]  = 8'd139;
    send_data[1]  = 8'd171;
    send_data[2]  = 8'd4;
    send_data[3]  = 8'd91;
    send_data[4]  = 8'd6;
    send_data[5]  = 8'd127;
    send_data[6]  = 8'd91;
    send_data[7]  = 8'd0;
    send_data[8]  = 8'd0;
    send_data[9]  = 8'd91;
    send_data[10] = 8'd3;
    send_data[11] = 8'd3;
    send_data[12] = 8'd107;
    send_data[13] = 8'd6;
    send_data[14] = 8'd65;
    send_data[15] = 8'd91;
    send_data[16] = 8'd0;
    send_data[17] = 8'd1;
    send_data[18] = 8'd91;
    send_data[19] = 8'd4;
    send_data[20] = 8'd1;
    send_data[21] = 8'd91;
    send_data[22] = 8'd2;
    send_data[23] = 8'd1;
    send_data[24] = 8'd124;
    send_data[25] = 8'd2;
    send_data[26] = 8'd0;
    send_data[27] = 8'd0;
    send_data[28] = 8'd0;
    send_data[29] = 8'd108;
    send_data[30] = 8'd6;
    send_data[31] = 8'd0;
    send_data[32] = 8'd92;
    send_data[33] = 8'd0;
    send_data[34] = 8'd1;
    send_data[35] = 8'd91;
    send_data[36] = 8'd1;
    send_data[37] = 8'd1;
end

task usb_host_send;
    input [7:0] pid;
    input [6:0] dev;
    input [3:0] ep;
    begin

    token_pid = pid;
    token_dev = dev;
    token_ep = {ep[0],ep[1],ep[2],ep[3]};
    $display("\n  pid = %h  dev = %h  ep=%h", pid, dev, ep);
    transfer_start_q <= 1'b1;
#20
    transfer_start_q <= 1'b0;
    end

endtask
usbh_sie
#( .USB_CLK_FREQ(60000000) )
u_sie
(
    // Clock & reset
    .clk_i(clk_60),
    .rst_i(reset),
    // Control
    .start_i(transfer_start_q),
    .in_transfer_i(in_transfer),
    .sof_transfer_i(sof_transfer_q),
    .resp_expected_i(1'b0),
    .ack_o(),//transfer_ack_w
    // Token packet
    .token_pid_i(token_pid),
    .token_dev_i(token_dev),
    .token_ep_i (token_ep),
    // Data packet
    .data_len_i(data_len),
    .data_idx_i(0),
    // Tx Data FIFO
    .tx_data_i(tx_data),
    .tx_pop_o(tx_pop),
    // Rx Data FIFO
    .rx_data_o(rx_data),
    .rx_push_o(rx_push),
    // Status
    .rx_done_o(rx_done),
    .tx_done_o(tx_done),
    .crc_err_o(),
    .timeout_o(),
    .response_o(),
    .rx_count_o(),
    .idle_o(),
    // UTMI Interface
    .utmi_data_o     (host_utmi_data_out ),
    .utmi_txvalid_o  (host_utmi_txvalid  ),
    .utmi_txready_i  (host_utmi_txready  ),
    .utmi_data_i     (host_utmi_data_in  ),
    .utmi_rxvalid_i  (host_utmi_rxvalid  ),
    .utmi_rxactive_i (host_utmi_rxactive ),
    .utmi_linestate_i(host_utmi_linestate)
);
//==============================================================
//======USB Host PHY
wire usb_dp_io;
wire usb_dn_io;
USB_SoftPHY_Top USB_SoftPHY_Host
(
     .clk_i             (clk_60              )
    ,.rst_i             (reset               )
    ,.utmi_data_out_i   (host_utmi_data_out  )
    ,.utmi_txvalid_i    (host_utmi_txvalid   )
    ,.utmi_data_in_o    (host_utmi_data_in   )
    ,.utmi_txready_o    (host_utmi_txready   )
    ,.utmi_rxvalid_o    (host_utmi_rxvalid   )
    ,.utmi_rxactive_o   (host_utmi_rxactive  )
    ,.utmi_rxerror_o    (host_utmi_rxerror   )
    ,.utmi_linestate_o  (host_utmi_linestate )
    ,.utmi_op_mode_i    (2'b01               )
    ,.utmi_xcvrselect_i (2'b01               )
    ,.utmi_termselect_i (1'b1                )
    // USB D+ / D-
    ,.usb_dp_io         (usb_dp_io)
    ,.usb_dn_io         (usb_dn_io)
);

pullup(usb_dp_io);
pulldown(usb_dn_io);

Top u_Top(
//interconnection
     .CLK_IN    (clk_in   )
    ,.RESET_N   (!reset   )
    ,.USB_DP    (usb_dp_io)
    ,.USB_DM    (usb_dn_io)
);


endmodule
